#!/usr/bin/env python2.7

# Version: 0.2
# Generic TCP Server for iSpindel (https://github.com/universam1/iSpindel)
# Receives iSpindel data as JSON via TCP socket and writes it to a CSV file, Database and/or Ubidots
# This is my first Python script ever, so please bear with me for now.
# Stephan Schreiber <stephan@sschreiber.de>, 2017-02-01

from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from datetime import datetime
import thread
import json

# CONFIG Start

# General
DEBUG = 0                               # Set to 1 to enable debug output on console
PORT = 9501                             # TCP Port to listen to
HOST = '0.0.0.0'                        # Allowed IP range. Leave at 0.0.0.0 to allow connections from anywhere

# CSV
CSV = 0                                 # Set to 1 if you want CSV (text file) output
OUTPATH = '/home/xbian/iSpindel/csv/'   # CSV output file path; filename will be name_id.csv
DELIMITER = ';'                         # CSV delimiter (normally use ; for Excel)
NEWLINE='\r\n'                          # newline (\r\n for windows clients)
DATETIME = 1                            # Leave this at 1 to include Excel compatible timestamp in CSV

# MySQL
SQL = 1                                 # 1 to enable output to MySQL database
SQL_DB = 'iSpindle'                     # Database name
SQL_USER = 'iSpindle'                   # DB user
SQL_PASSWORD = 'xxxxxx'                 # DB user's password (change this)

# Ubidots (using existing account)
UBIDOTS = 1                                     # 1 to enable output to ubidots
UBI_TOKEN = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'    # ubidots token, see manual

# CONFIG End

ACK = chr(6)            # ASCII ACK (Acknowledge)
NAK = chr(21)           # ASCII NAK (Not Acknowledged)
BUFF = 1024             # Buffer Size (greatly exaggerated for now)

def dbgprint(s):
    if DEBUG == 1: print(s)

def handler(clientsock,addr):
    inpstr = ''
    success = False
    spindle_name = ''
    spindle_id = ''
    angle = 0.0
    temperature = 0.0
    battery = 0.0
    while 1:
        data = clientsock.recv(BUFF)
        if not data: break  # client closed connection
        dbgprint(repr(addr) + ' received:' + repr(data))
        if "close" == data.rstrip():
            clientsock.send(ACK)
            dbgprint(repr(addr) + ' ACK sent. Closing.')
            break   # close connection
        try:
            inpstr += str(data.rstrip())
            if inpstr[0] != "{" :
                clientsock.send(NAK)
                dbgprint(repr(addr) + ' Not JSON.')
                break # close connection
            dbgprint(repr(addr) + ' Input Str is now:' + inpstr)
            if inpstr.find("}") != -1 :
                jinput = json.loads(inpstr)
                spindle_name = jinput['name']
                spindle_id = jinput['ID']
                angle = jinput['angle']
                temperature = jinput['temperature']
                battery = jinput['battery']
                # looks like everything went well :)
                clientsock.send(ACK)
                success = True
                dbgprint(repr(addr) + ' ' + spindle_name + ' (ID:' + spindle_id + ') : Data received OK.')
                break # close connection
        except Exception as e:
            # something went wrong
            # traceback.print_exc() # this would be too verbose, so let's do this instead:
            dbgprint(repr(addr) + ' Error: ' + str(e))
            clientsock.send(NAK)
            dbgprint(repr(addr) + ' NAK sent.')
            break # close connection server side after non-success
    clientsock.close()
    dbgprint(repr(addr) + " - closed connection") #log on console

    if success :
        # We have the complete spindle data now, so let's make it available
        if CSV == 1:
            dbgprint(repr(addr) + ' - writing CSV')
            try:
                filename = OUTPATH + spindle_name + '_' + spindle_id + '.csv'
                with open(filename, 'a') as csv_file:
                    # this would sort output. But we do not want that...
                    # csvw = csv.writer(csv_file, delimiter=DELIMITER)
                    # csvw.writerow(jinput.values())
                    outstr = ''
                    outstr += spindle_name + DELIMITER
                    outstr += spindle_id + DELIMITER
                    outstr += str(angle) + DELIMITER
                    outstr += str(temperature) + DELIMITER
                    outstr += str(battery)
                    if DATETIME == 1:
                        cdt = datetime.now()
                        outstr += DELIMITER + cdt.strftime('%x %X')
                    outstr += NEWLINE
                    csv_file.writelines(outstr)
                    dbgprint(repr(addr) + ' - CSV data written.')
            except Exception as e:
                dbgprint(repr(addr) + ' CSV Error: ' + str(e))

        if SQL == 1:
            try:
                dbgprint(repr(addr) + ' - writing to database')
                import mysql.connector
                cnx = mysql.connector.connect(user=SQL_USER, password=SQL_PASSWORD, host='127.0.0.1', database=SQL_DB)
                cur = cnx.cursor()
                add_sql = 'INSERT INTO Data (Timestamp, Name, ID, Angle, Temperature, Battery)'
                add_sql += ' VALUES (%s, %s, %s, %s, %s, %s)'
                add_data = (datetime.now(), spindle_name, spindle_id, angle, temperature, battery)
                cur.execute(add_sql, add_data)
                cnx.commit()
                cur.close()
                cnx.close()
                dbgprint(repr(addr) + ' - DB data written.')
            except Exception as e:
                dbgprint(repr(addr) + ' Database Error: ' + str(e))

        if UBIDOTS == 1:
            try:
                dbgprint(repr(addr) + ' - sending to ubidots')
                import urllib2
                outdata = {
                    'tilt' : angle,
                    'temperature' : temperature,
                    'battery' : battery
                }
                out = json.dumps(outdata)
                dbgprint(repr(addr) + ' - sending: ' + out)
                url = 'http://things.ubidots.com/api/v1.6/devices/' + spindle_name + '?token=' + UBI_TOKEN
                req = urllib2.Request(url)
                req.add_header('Content-Type', 'application/json')
                req.add_header('User-Agent', spindle_name)
                response = urllib2.urlopen(req, out)
                dbgprint(repr(addr) + ' - received: ' + str(response))
            except Exception as e:
                dbgprint(repr(addr) + ' Ubidots Error: ' + str(e))


def main():
    ADDR = (HOST, PORT)
    serversock = socket(AF_INET, SOCK_STREAM)
    serversock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    serversock.bind(ADDR)
    serversock.listen(5)
    while 1:
        dbgprint('waiting for connection... listening on port: ' + str(PORT))
        clientsock, addr = serversock.accept()
        dbgprint('...connected from: ' + str(addr))
        thread.start_new_thread(handler, (clientsock, addr))

if __name__ == "__main__":
    main()
